﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Phidget22;
using Phidget22.Events;
using Phidget22.ExampleUtils;

namespace Gyroscope_Example {
	public partial class Form1 : Form {
		CommandLineOpen open;
		formCleaner cleaner;
		Gyroscope gyroscope = null;
		private TemperatureSensor temperatureSensor;
		private ErrorEventBox errorBox;
		private DataGraphBox xGraphBox, yGraphBox, zGraphBox;

		double[] lastMsCount = { 0, 0, 0 };
		bool[] lastMsCountGood = { false, false, false };
		double[] gyroHeading = { 0, 0, 0 }; //degrees

		public Form1() {
			open = new CommandLineOpen(this);
			cleaner = new formCleaner(this);
			InitializeComponent();
		}

		public Form1(String[] commandLine) {
			open = new CommandLineOpen(this);
			cleaner = new formCleaner(this);
			open.commandLine = commandLine;
			InitializeComponent();
		}

		private void Form1_Load(object sender, EventArgs e) {

			errorBox = new ErrorEventBox(this);
			gyroscope = new Gyroscope();

			xGraphBox = new DataGraphBox(this, "°/s");
			yGraphBox = new DataGraphBox(this, "°/s");
			zGraphBox = new DataGraphBox(this, "°/s");

			//register necessary handlers
			gyroscope.Attach += gyroscope_attach;
			gyroscope.Detach += gyroscope_detach;
			gyroscope.Error += gyroscope_error;
			gyroscope.PropertyChange += gyroscope_propertychange;

			// This causes the handler to get called without Invoke
			FastEventHandler<GyroscopeAngularRateUpdateEventArgs> rateEventHandler = new FastEventHandler<GyroscopeAngularRateUpdateEventArgs>();
			gyroscope.AngularRateUpdate += rateEventHandler.Handler;
			rateEventHandler.Event += angularRate_update;

			temperatureSensor = new TemperatureSensor();
			temperatureSensor.TemperatureChange += TemperatureSensor_TemperatureChange;
			temperatureSensor.Detach += TemperatureSensor_Detach;

			//open the hardware for use
			commandLineData phidgetParameters = open.parseCmdLine(); //get command line parameters

			try { //set all the values grabbed from command line.  these values have defaults that are set in ExampleUtils.cs, you can check there to see them.
				gyroscope.Channel = phidgetParameters.Channel; //selects the channel on the device to open
				gyroscope.DeviceSerialNumber = phidgetParameters.SerialNumber; //selects the device or hub to open
				gyroscope.HubPort = phidgetParameters.HubPort; //selects the port on the hub to open
				gyroscope.IsHubPortDevice = phidgetParameters.isHubPortDevice; //is the device a port on a VINT hub?

				if (phidgetParameters.isRemote) //are we trying to open a remote device?
				{
					gyroscope.IsRemote = true;
					Net.EnableServerDiscovery(ServerType.Device); //turn on network scan
					if (phidgetParameters.Password != null && phidgetParameters.ServerName != null)
						Net.SetServerPassword(phidgetParameters.ServerName, phidgetParameters.Password); //set the password if there is one
				} else
					gyroscope.IsLocal = true;

				gyroscope.Open(); //open the device specified by the above parameters
			} catch (PhidgetException ex) { errorBox.addMessage("Error opening device: " + ex.Message); }

		}

		private void gyroscope_propertychange(object sender, PropertyChangeEventArgs e)
		{

			switch (e.PropertyName)
			{
				case "DataInterval":
					dataIntervalTrk.Value = gyroscope.DataInterval;
					break;
				case "DataRate":
					dataIntervalTrk.Value = gyroscope.DataRate;
					break;
			}
		}
		private void TemperatureSensor_Detach(object sender, DetachEventArgs e)
		{
			temperatureReading.Text = "Detached";
		}

		private void TemperatureSensor_TemperatureChange(object sender, TemperatureSensorTemperatureChangeEventArgs e)
		{
			temperatureReading.Text = e.Temperature.ToString() + "°C";
			if (e.Temperature > 44.8 && e.Temperature < 45.2)
				temperatureReading.ForeColor = Color.Green;
			else
				temperatureReading.ForeColor = Color.Red;
		}

		private void Form1_FormClosing(object sender, FormClosingEventArgs e) {
			gyroscope.Close();
			temperatureSensor.Close();
		}

		void gyroscope_attach(object sender, Phidget22.Events.AttachEventArgs e) {
			phidgetInfoBox1.FillPhidgetInfo((Phidget)sender);
			Gyroscope attachedDevice = (Gyroscope)sender;

			xGraphBox.SetTitle(DataGraphBox.GenerateTitle("X Axis", (Phidget)attachedDevice));
			yGraphBox.SetTitle(DataGraphBox.GenerateTitle("Y Axis", (Phidget)attachedDevice));
			zGraphBox.SetTitle(DataGraphBox.GenerateTitle("Z Axis", (Phidget)attachedDevice));

			settingsBox.Visible = true;
			outputBox.Visible = true;
			gaugeBox.Visible = true;

			gyroHeading[0] = 0;
			gyroHeading[1] = 0;
			gyroHeading[2] = 0;
			xPosition.Value = 0;
			yPosition.Value = 0;
			zPosition.Value = 0;

			switch (attachedDevice.DeviceID)
			{
				case DeviceID.PN_MOT0109:
					heatingEnabledPanel.Visible = true;
					heatingEnabledBox.Checked = attachedDevice.HeatingEnabled;

					//Use the temperature sensor specifically from the attached device
					try
					{ //set all the values grabbed from command line.  these values have defaults that are set in ExampleUtils.cs, you can check there to see them.
						temperatureSensor.Channel = attachedDevice.Channel; //selects the channel on the device to open
						temperatureSensor.DeviceSerialNumber = attachedDevice.DeviceSerialNumber; //selects the device or hub to open
						temperatureSensor.HubPort = attachedDevice.HubPort; //selects the port on the hub to open
						temperatureSensor.IsHubPortDevice = attachedDevice.IsHubPortDevice; //is the device a port on a VINT hub?

						if (attachedDevice.IsRemote) //are we trying to open a remote device?
						{
							commandLineData phidgetParameters = open.parseCmdLine(); //get command line parameters for network info
							temperatureSensor.IsRemote = true;
							Net.EnableServerDiscovery(ServerType.Device); //turn on network scan
							if (phidgetParameters.Password != null && phidgetParameters.ServerName != null)
								Net.SetServerPassword(phidgetParameters.ServerName, phidgetParameters.Password); //set the password if there is one
						}
						else
							temperatureSensor.IsLocal = true;

						temperatureSensor.Open(); //open the device specified by the above parameters
					}
					catch (PhidgetException ex) { errorBox.addMessage("Error opening device: " + ex.Message); }

					break;
				default:
					heatingEnabledPanel.Visible = false;
					break;
			}

			try {//these parameters must be set or the device won't function
				attachedDevice.DataInterval = 32; //setting the interval lower than this is possible but it can cause the GUI to bog down
				dataIntervalTrk.Unit = "ms";
				dataIntervalTrk.minTxt = attachedDevice.MinDataInterval.ToString();
				dataIntervalTrk.Minimum = attachedDevice.MinDataInterval;
				dataIntervalTrk.maxTxt = attachedDevice.MaxDataInterval.ToString();
				dataIntervalTrk.Maximum = attachedDevice.MaxDataInterval;
				dataIntervalTrk.Value = 32;
				dataIntervalTrk.trkText = attachedDevice.DataInterval.ToString();
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error setting data interval: " + ex.Message);
			}

			guiUpdateTimer.Start();
		}


		void gyroscope_detach(object sender, Phidget22.Events.DetachEventArgs e) {
			guiUpdateTimer.Stop();
			_rate[0] = 0;
			_rate[1] = 0;
			_rate[2] = 0;
			_timestamp = 0;

			phidgetInfoBox1.Clear();
			cleaner.clean();

			temperatureSensor.Close();

			settingsBox.Visible = false;
			outputBox.Visible = false;
			gaugeBox.Visible = false;
		}
		void gyroscope_error(object sender, Phidget22.Events.ErrorEventArgs e) {
			errorBox.addMessage(e.Description);

			if (e.Code == ErrorEventCode.Saturation) {
				xGraphBox.addGap();
				yGraphBox.addGap();
				zGraphBox.addGap();
			}
		}

		double[] _rate = { 0, 0, 0 };
		double _timestamp = 0;
		private void guiUpdateTimer_Tick(object sender, EventArgs e) {

			xRatetxt.Text = _rate[0].ToString() + " °/s";
			yRatetxt.Text = _rate[1].ToString() + " °/s";
			zRatetxt.Text = _rate[2].ToString() + " °/s";
			timestamptxt.Text = _timestamp.ToString() + " ms";

			xPosition.Value = (float)gyroHeading[0];
			yPosition.Value = (float)gyroHeading[1];
			zPosition.Value = (float)gyroHeading[2];
		}

		void angularRate_update(object sender, Phidget22.Events.GyroscopeAngularRateUpdateEventArgs e) {
			try {//display the angular rates and calculate the angular positions
				calculateGyroHeading(e, 0);
				calculateGyroHeading(e, 1);
				calculateGyroHeading(e, 2);

				_rate[0] = e.AngularRate[0];
				_rate[1] = e.AngularRate[1];
				_rate[2] = e.AngularRate[2];
				_timestamp = e.Timestamp;

				xGraphBox.addData(e.AngularRate[0], gyroscope.DataInterval, e.Timestamp);
				yGraphBox.addData(e.AngularRate[1], gyroscope.DataInterval, e.Timestamp);
				zGraphBox.addData(e.AngularRate[2], gyroscope.DataInterval, e.Timestamp);

			} catch (PhidgetException ex) {
				errorBox.addMessage("Error reading output: " + ex.Message);
			}
		}

		private void dataIntervalTrk_Scroll(object sender, EventArgs e) {
			if (MouseButtons != MouseButtons.Left) {
				try {
					gyroscope.DataInterval = (int)dataIntervalTrk.Value;
				} catch (Exception ex) {
					errorBox.addMessage("Error setting data interval: " + ex.Message);
				}
			}
		}

		private void updateGraphDataInterval()
		{
			try
			{
				xGraphBox.DataInterval = gyroscope.DataInterval;
				yGraphBox.DataInterval = gyroscope.DataInterval;
				zGraphBox.DataInterval = gyroscope.DataInterval;
			}
			catch (PhidgetException ex) { errorBox.addMessage("Error updating data interval: " + ex.Message); }
		}
		private void dataIntervalTrk_LogMouseUp(object sender, MouseEventArgs e) {
			if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL)
			{
				try
				{
					gyroscope.DataInterval = (int)Math.Round(dataIntervalTrk.Value);
					updateGraphDataInterval();
				}
				catch (Exception ex)
				{
					errorBox.addMessage("Error setting data interval: " + ex.Message);
				}
			}
			else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE)
			{
				try
				{
					gyroscope.DataRate = dataIntervalTrk.Value;
					updateGraphDataInterval();
				}
				catch (Exception ex)
				{
					errorBox.addMessage("Error setting data rate: " + ex.Message);
				}
			}
		}

		void calculateGyroHeading(Phidget22.Events.GyroscopeAngularRateUpdateEventArgs data, int index) {
			double gyro = 0;
			gyro = data.AngularRate[index];

			if (lastMsCountGood[index]) {//calculate heading
				double timechange = data.Timestamp - lastMsCount[index]; // in ms
				double timeChangeSeconds = (double)timechange / 1000.0;

				gyroHeading[index] += timeChangeSeconds * gyro;

				//ensure proper wrap around
				if (gyroHeading[index] > 180)
					gyroHeading[index] -= 360;
				else if (gyroHeading[index] < -180)
					gyroHeading[index] += 360;
			}

			lastMsCount[index] = data.Timestamp;
			lastMsCountGood[index] = true;
		}

		private void zeroBtn_Click(object sender, EventArgs e) {//zero the gyroscope values
			gyroscope.Zero();
			System.Threading.Thread.Sleep(100);
			gyroHeading[0] = 0;
			gyroHeading[1] = 0;
			gyroHeading[2] = 0;
		}

		private void xGraphBtn_Click(object sender, EventArgs e) {
			if (!xGraphBox.Visible)
				xGraphBox.Show();
		}

		private void yGraphBtn_Click(object sender, EventArgs e) {
			if (!yGraphBox.Visible)
				yGraphBox.Show();
		}

		private void zGraphBtn_Click(object sender, EventArgs e) {
			if (!zGraphBox.Visible)
				zGraphBox.Show();
		}

		private void heatingEnabledBox_CheckedChanged(object sender, EventArgs e)
		{
			gyroscope.HeatingEnabled = heatingEnabledBox.Checked;
		}

        private void dataIntervalTrk_ModeSwitch(object sender, EventArgs e)
        {
			dataIntervalTrk.SwitchModes();
			if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE)
			{
				try
				{
					dataIntervalTrk.isIntValue = false;
					dataIntervalTrk.Unit = "Hz";
					dataIntervalTrk.Maximum = gyroscope.MaxDataRate;
					dataIntervalTrk.Minimum = gyroscope.MinDataRate;
					dataIntervalTrk.maxTxt = gyroscope.MaxDataRate.ToString();
					dataIntervalTrk.minTxt = gyroscope.MinDataRate.ToString("F3");
					dataIntervalTrk.trkText = gyroscope.DataRate.ToString() + "Hz";
					dataIntervalTrk.Value = gyroscope.DataRate;
				}
				catch (Exception ex)
				{ errorBox.addMessage("Error setting data rate: " + ex.Message); }

			}
			else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL)
			{
				try
				{
					dataIntervalTrk.isIntValue = true;
					dataIntervalTrk.Unit = "ms";
					dataIntervalTrk.Maximum = gyroscope.MaxDataInterval;
					dataIntervalTrk.Minimum = gyroscope.MinDataInterval;
					dataIntervalTrk.maxTxt = gyroscope.MaxDataInterval.ToString();
					dataIntervalTrk.minTxt = gyroscope.MinDataInterval.ToString();
					dataIntervalTrk.trkText = gyroscope.DataInterval.ToString() + "ms";
					dataIntervalTrk.Value = gyroscope.DataInterval;
				}
				catch (Exception ex)
				{ errorBox.addMessage("Error setting data interval: " + ex.Message); }
			}
		}
		private void dataIntervalTrk_SetClick(object sender, EventArgs e)
		{
			if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL)
			{
				try
				{
					gyroscope.DataInterval = (int)dataIntervalTrk.Value;
					updateGraphDataInterval();
				}
				catch (Exception ex) { errorBox.addMessage("Error setting data interval: " + ex.Message); }
			}
			else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE)
			{
				try
				{
					gyroscope.DataRate = dataIntervalTrk.Value;
					updateGraphDataInterval();
				}
				catch (Exception ex) { errorBox.addMessage("Error setting data rate: " + ex.Message); }
			}

		}

		private void dataIntervalTrk_SetEnter(object sender, KeyEventArgs e)
		{
			if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL)
			{
				if (e.KeyCode == Keys.Enter)
				{
					try
					{
						gyroscope.DataInterval = (int)dataIntervalTrk.Value;
						updateGraphDataInterval();
					}
					catch (Exception ex) { errorBox.addMessage("Error setting data interval: " + ex.Message); }
				}
			}
			else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE)
			{
				try
				{
					gyroscope.DataRate = dataIntervalTrk.Value;
					updateGraphDataInterval();
				}
				catch (Exception ex) { errorBox.addMessage("Error setting data rate: " + ex.Message); }
			}
		}
	}
}